/*
 * Copyright (c) 2018 Koji Suzuki
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define I2CKBD_ADDR	0x44

#define LOCALDESC_REGISTER	0
#define HIDDESC_REGISTER	1
#define REPORTDESC_REGISTER	2
#define INPUT_REGISTER		3
#define OUTPUT_REGISTER		4
#define COMMAND_REGISTER	5
#define DATA_REGISTER		6
#define KEYSTAT_REGISTER	7

#define CMD_RESET		1
#define CMD_GET_REPORT		2
#define CMD_SET_POWER		8
#define CMD_VENDOR		14


#define CMD_SET_REPORT		3
#define CMD_GET_IDLE		4
#define CMD_SET_IDLE		5
#define CMD_GET_PROTOCOL	6
#define CMD_SET_PROTOCOL	7

#define F_TIMER_TICK0	(TIFR0 & (1<<OCF0A))
#define F_TIMER_TICK	(GPIOR0 & (1<<0))
#define F_REPEAT	(GPIOR0 & (1<<1))
#define F_REPEAT2	(GPIOR0 & (1<<2))
#define F_MODE_CHG	(GPIOR0 & (1<<3))
#define F_STOP_SCAN	(GPIOR0 & (1<<4))
#define F_BUILD_REQ	(GPIOR0 & (1<<5))
#define F_BUILD_DONE	(GPIOR0 & (1<<6))
#define F_PGM_DATA	(GPIOR0 & (1<<7))

#define clear_F_TIMER_TICK0()	{GPIOR0 |= (1<<OCF0A);}
#define clear_F_TIMER_TICK()	{GPIOR0 &= ~(1<<0);}
#define clear_F_REPEAT()	{GPIOR0 &= ~(1<<1);}
#define clear_F_REPEAT2()	{GPIOR0 &= ~(1<<2);}
#define clear_F_MODE_CHG()	{GPIOR0 &= ~(1<<3);}
#define clear_F_STOP_SCAN()	{GPIOR0 &= ~(1<<4);}
#define clear_F_BUILD_REQ()	{GPIOR0 &= ~(1<<5);}
#define clear_F_BUILD_DONE()	{GPIOR0 &= ~(1<<6);}
#define clear_F_PGM_DATA()	{GPIOR0 &= ~(1<<7);}

#define set_F_TIMER_TICK()	{GPIOR0 |= (1<<0);}
#define set_F_REPEAT()		{GPIOR0 |= (1<<1);}
#define set_F_REPEAT2()		{GPIOR0 |= (1<<2);}
#define set_F_MODE_CHG()	{GPIOR0 |= (1<<3);}
#define set_F_STOP_SCAN()	{GPIOR0 |= (1<<4);}
#define set_F_BUILD_REQ()	{GPIOR0 |= (1<<5);}
#define set_F_BUILD_DONE()	{GPIOR0 |= (1<<6);}
#define set_F_PGM_DATA()	{GPIOR0 |= (1<<7);}

#define F_FN		(GPIOR2 & (1<<0))
#define F_CTRL		(GPIOR2 & (1<<1))
#define F_SHIFT		(GPIOR2 & (1<<2))
#define F_ALT		(GPIOR2 & (1<<3))
#define clear_F_FN()	{GPIOR2 &= ~(1<<0);}
#define clear_F_CTRL()	{GPIOR2 &= ~(1<<1);}
#define clear_F_SHIFT()	{GPIOR2 &= ~(1<<2);}
#define clear_F_ALT()	{GPIOR2 &= ~(1<<3);}
#define set_F_FN()	{GPIOR2 |= (1<<0);}
#define set_F_CTRL()	{GPIOR2 |= (1<<1);}
#define set_F_SHIFT()	{GPIOR2 |= (1<<2);}
#define set_F_ALT()	{GPIOR2 |= (1<<3);}

#define TYPE_2313	0
#define TYPE_861	1
#define TYPE_T88A	2
#define TYPE_T88B	3
#define TYPE_T88C	4
#define TYPE_T88D	5
#define TYPE_T88E	6


#define KBD_TYPE	TYPE_2313
//#define KBD_TYPE	TYPE_861
//#define KBD_TYPE	TYPE_T88E
#if defined(__AVR_ATtiny861__) || defined(__AVR_ATtiny861A__)
#undef KBD_TYPE
#define KBD_TYPE	TYPE_861
#endif
#if defined(__AVR_ATtiny2313__) || defined(__AVR_ATtiny2313A__) || defined(__AVR_ATtiny4313__)
#undef KBD_TYPE
#define KBD_TYPE	TYPE_2313
#endif


#define STACK_TUNE
#define USE_GPIOR1
#define USE_GPIOR2
#define USE_SLEEP
#define FN_EXTEND

//#define SUPPORT_LOCALCODE
//#define SUPPORT_ASCIICODE
//#define SUPPORT_SCANCODE1
#define SUPPORT_HIDCODE

extern uint8_t read_buf_len;
extern uint8_t read_buf[10];
extern uint8_t keystat_tab[];
extern volatile uint8_t kbd_mode;

void i2c_init();
